unit ms_mouse;

interface

type
   grcurs = record
               sc_mask : array[0..15] of word;
               cs_mask : array[0..15] of word;
            end;
var
   mick_x : word;
   mick_y : word;

{ *** Procedure and Function headings from implementation section *** }

function mouse_present:boolean           { Determines if mouse is present };
function num_buttons:byte                { Get number of buttons on mouse };
procedure show_cursor                    { Display mouse cursor };
procedure hide_cursor                    { Hide mouse cursor };
procedure read_status(var x,y,b:word)    { Return status };
function get_x:word                      { Return x position };
function get_y:word                      { Return y position };
procedure position_cursor(x,y:word)      { Move cursor to position };
procedure button_pressed(      b:byte;   { Button to check }
                         var w,t:word;   { Which Pressed / Times specified }
                         var x,y:word)   { Where when specified last pressed };
procedure button_release(      b:byte;   { Button to check }
                         var w,t:word;   { Which released / Times specified }
                         var x,y:word)   { Where when specified last pressed };
procedure set_x_range(x1,x2:word)        { Min and max x range };
procedure set_y_range(y1,y2:word)        { Min and max y range };
procedure define_gr_cursor(  x,y:word;   { Hot spot location }
                           var l:grcurs) { bitmap 32 words };
procedure define_tx_cursor(soft:boolean; { True for software cursor }
                            t,b:word);
procedure read_motion(var x,y:word)       { Read mickeys since last call };
procedure emulate_lp_on                  { Light Pen Emulation on };
procedure emulate_lp_off                 { Light Pen Emulation on };
procedure mickey_pel_ratio(x,y:word)     { mickeys per 8 pixels };

implementation

uses
   dos;

const
   mouse_int = $33;

var
   r  : registers;
   m_ : boolean;

function mouse_present:boolean           { Determines if mouse is present };
   begin
      r.ax := $0000;
      intr(mouse_int,r);
      if r.ax = $0000 then
         mouse_present := false
      else
         mouse_present := true;
   end;

function num_buttons:byte                { Get number of buttons on mouse };
   begin
      if m_ then
         begin
            r.ax := $0000;
            intr(mouse_int,r);
            case r.bl of
               $00 : num_buttons := 0;
               $03 : num_buttons := 3;
               $FF : num_buttons := 2;
               else  num_buttons := 1;
            end;
         end
      else
         num_buttons := 0;
   end;

procedure show_cursor                    { Display mouse cursor };
   begin
      if m_ then
         begin
            r.ax := $0001;
            intr(mouse_int,r);
         end;
   end;

procedure hide_cursor                    { Hide mouse cursor };
   begin
      if m_ then
         begin
            r.ax := $0002;
            intr(mouse_int,r);
         end;
   end;

procedure read_status(var x,y,b:word)    { Return status };
   begin
      if m_ then
         begin
            r.ax := $0003;
            intr(mouse_int,r);
            x := r.cx;
            y := r.dx;
            b := r.bx;
         end
      else
         begin
            x := $0000;
            y := $0000;
            b := $0000;
         end;
   end;

function get_x:word                      { Return x position };
   begin
      if m_ then
         begin
            r.ax := $0003;
            intr(mouse_int,r);
            get_x := r.cx;
         end
      else
         get_x := $0000;
   end;

function get_y:word                      { Return y position };
   begin
      if m_ then
         begin
            r.ax := $0003;
            intr(mouse_int,r);
            get_y := r.dx;
         end
      else
         get_y := $0000;
   end;

procedure position_cursor(x,y:word)      { Move cursor to position };
   begin
      if m_ then
         begin
            r.ax := $0004;
            r.cx := x;
            r.dx := y;
            intr(mouse_int,r);
         end;
   end;

procedure button_pressed(      b:byte;   { Button to check }
                         var w,t:word;   { Which Pressed / Times specified }
                         var x,y:word)   { Where when specified last pressed };
   begin
      if m_ then
         begin
            r.ax := $0005;
            r.bh := $00;
            r.bl := b;
            intr(mouse_int,r);
            w := r.ax;
            t := r.bx;
            x := r.cx;
            y := r.dx;
         end
      else
         begin
            w := $0000;
            t := $0000;
            x := $0000;
            y := $0000;
         end;
   end;

procedure button_release(      b:byte;   { Button to check }
                         var w,t:word;   { Which released / Times specified }
                         var x,y:word)   { Where when specified last pressed };
   begin
      if m_ then
         begin
            r.ax := $0006;
            r.bh := $00;
            r.bl := b;
            intr(mouse_int,r);
            w := r.ax;
            t := r.bx;
            x := r.cx;
            y := r.dx;
         end
      else
         begin
            w := $0000;
            t := $0000;
            x := $0000;
            y := $0000;
         end;
   end;

procedure set_x_range(x1,x2:word)        { Min and max x range };
   begin
      if m_ then
         begin
            r.ax := $0007;
            r.cx := x1;
            r.dx := x2;
            intr(mouse_int,r);
         end;
   end;

procedure set_y_range(y1,y2:word)        { Min and max y range };
   begin
      if m_ then
         begin
            r.ax := $0008;
            r.cx := y1;
            r.dx := y2;
            intr(mouse_int,r);
         end;
   end;

procedure define_gr_cursor(  x,y:word;   { Hot spot location }
                           var l:grcurs) { bitmap 32 words };
   begin
      if m_ then
         begin
            r.ax := $0009;
            r.bx := x;
            r.cx := y;
            r.es := seg(l);
            r.dx := ofs(l);
            intr(mouse_int,r);
         end;
   end;

procedure define_tx_cursor(soft:boolean; { True for software cursor }
                            t,b:word);
   begin
      if m_ then
         begin
            r.ax := $000A;
            r.cx := t;
            r.dx := b;
            if soft then
               r.bx := $0000
            else
               r.bx := $0001;
            intr(mouse_int,r);
         end;
   end;

procedure read_motion(var x,y:word)       { Read mickeys since last call };
   begin
      if m_ then
         begin
            r.ax := $000C;
            intr(mouse_int,r);
            x := r.cx;
            y := r.dx;
         end
      else
         begin
            x := $0000;
            y := $0000;
         end;
   end;

procedure emulate_lp_on                  { Light Pen Emulation on };
   begin
      if m_ then
         begin
            r.ax := $000D;
            intr(mouse_int,r);
         end;
   end;

procedure emulate_lp_off                 { Light Pen Emulation on };
   begin
      if m_ then
         begin
            r.ax := $000E;
            intr(mouse_int,r);
         end;
   end;

procedure mickey_pel_ratio(x,y:word)     { mickeys per 8 pixels };
   begin
      if m_ then
         begin
            r.ax := $000F;
            r.cx := x;
            r.dx := y;
            intr(mouse_int,r);
            mick_x := x;
            mick_y := y;
         end
      else
         begin
            mick_x := $0001;
            mick_y := $0001;
         end;
   end;

begin
   if mouse_present then
      begin
         m_ := true;
         set_x_range(0,79);
         set_y_range(0,24);
         mickey_pel_ratio(8,8);
         define_tx_cursor(false,0,13);
         show_cursor;
      end
   else
      m_ := false;
end.
